unit dynkenkomp;
{
  Purpose: Dynaamisesti list taulun kentti paneelin sislle
  Author:  Vesa Lappalainen
  Date:    21.08.1996
  Changes: 14.10.1999/vl
           + muutettu komponentiksi  ( kommentti komp )
  Usage:   Alustus dynaamisesti:
             olion varaus dynaamisesti
             kentat:TDynKentat;
             ...
             kentat := TDynKentat.Create(self);
           Komponettien asettaminen esim:
             kentat.DataSource := DataSourcePuh;
           Poisto:
             kentat.Free

           Komponentti ottaa kytettvn DataSetin AfterOpen ja BeforeClose
           tapahtumat itselleen ja muuttaa tarvittaessa lomakemuotoista
           esityst DataSetin muuttuessa.

           Normaalikytss (= vedetn orrelta) asetetaan ulkoiset ominaisuudet
           ja DataSource.
   ToDo:   - vlkkyminen pois, eli nytn pivitys pois Siivoa yms. ajaksi.        
}

interface

uses
  Classes,
{$ifdef CLX}
   QStdCtrls,  QDBCtrls, QControls,QExtctrls,Qforms,
{$else}
   StdCtrls,  DBCtrls, Controls,extctrls,forms,
{$endif}
   DB;

const MAX_KENTTIA       = 20;
      C_KENTAN_KORKEUS  = 0;
      C_NIMEN_LEVEYS    = 60;
      C_EDIT_LEVEYS     = 200;
      C_RIGHT_MARGIN    = 10;                                           { komp }

type
  {----------------------------------------------------------------------------}
  cKentta = object { Pari: otsikko - tietueen kentt }
    nimi : TLabel;
    edit : TDBEdit;
    index : integer;
  end;

  {----------------------------------------------------------------------------}
  TDynKentat = class(TScrollBox)                                        { komp }
  private
    kentat : array [0..MAX_KENTTIA] of cKentta; { Taulukko kentist            }
    n : integer;                                { Kenttien lukumr           }
    kentan_korkeus,
    nimen_leveys,
    edit_leveys:integer;
    FDataSource: TDataSource;
    FRightMargin: integer;
    FWorking : boolean; { Estetn rekursioita }                        { komp }
    FOldBefore : TDataSetNotifyEvent;                                   { komp }
    FOldAfter : TDataSetNotifyEvent;                                    { komp }
    procedure Paikka(var kentta:cKentta);       { Vaihtaa kentn paikan ja koon}
    procedure MuutaKoko;                        { Vaihtaa kaikk kenttien koot  }
    procedure Lisaa(s:string);                  { Lis yhden kentn           }
    procedure LisaaKaikki;
    procedure SetDataSource(const Value: TDataSource);    { Lis kaikki kentt          }
    procedure BeforeClose(DataSet: TDataSet);                           { komp }
    procedure AfterOpen(DataSet: TDataSet);                             { komp }
    procedure HandleResize;                                             { komp }
    procedure SetRightMargin(const Value: integer);                     { komp }
  public
    procedure Loaded; override;                                         { komp }
    constructor Create(AOwner:TComponent); override;                    { komp }
    destructor Destroy; override;
    procedure Siivoa;                           { Poistaa kaikki kentt        }
    procedure SetEditLeveys(l:integer);
    procedure SetNimenLeveys(l:integer);
    procedure SetKentanKorkeus(h:integer);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;{ komp }
  published                                                              { komp }
    property DataSource : TDataSource read FDataSource write SetDataSource;
    property EditLeveys:integer read edit_leveys write SetEditLeveys;
    property NimenLeveys:integer read nimen_leveys write SetNimenLeveys;
    property KentanKorkeus:integer read kentan_korkeus write SetKentanKorkeus;
    property RightMargin : integer read FRightMargin write SetRightMargin default C_RIGHT_MARGIN;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('GKOtest',[TDynKentat]);
end;



type                                                                    { komp }
  TSizeWinControl = class(TWinControl)
  public
    property OnResize;
  end;

{------------------------------------------------------------------------------}
constructor TDynKentat.Create(AOwner:TComponent);
begin
  inherited;
  n := 0;
  kentan_korkeus := C_KENTAN_KORKEUS;
  nimen_leveys   := C_NIMEN_LEVEYS;
  edit_leveys    := C_EDIT_LEVEYS;
  FRightMargin   := C_RIGHT_MARGIN;                                     { komp }
  Caption := ' ';                                                       { komp }
  HorzScrollBar.Visible := False;                                       { komp }
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.Siivoa;
var i:integer;
begin
  FWorking := true;                                                     { komp }
  for i:=0 to n-1 do begin
    kentat[i].nimi.Free; kentat[i].nimi := NIL;
    kentat[i].edit.Free; kentat[i].edit := NIL;
  end;
  n := 0;
  FWorking := false;                                                   { komp }
end;

{------------------------------------------------------------------------------}
destructor TDynKentat.Destroy;
begin
  Siivoa;
  DataSource := nil;                                                    { komp }
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.Paikka(var kentta:cKentta);
var korkeus:integer;
begin
  korkeus := kentan_korkeus;
  if ( korkeus <= 0 ) then korkeus := kentta.Edit.Height+1;
  with ( kentta.nimi ) do begin
    Left := 10;
    Width := nimen_leveys;
    Top := 10 + kentta.index*korkeus;
   end;
  with ( kentta.edit ) do begin
    Left := 10 + kentta.nimi.Left + nimen_leveys;
    Width := edit_leveys;
    Top := kentta.nimi.Top;
   end;
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.MuutaKoko;
var i:integer;
begin
  for i:=0 to n-1 do
    Paikka(kentat[i]);
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.Lisaa(s:string);
begin
  if ( DataSource = nil ) then exit;                                    { komp }
  if ( n >= MAX_KENTTIA ) then exit;
  kentat[n].nimi := TLabel.Create(self);                                { komp }
  with ( kentat[n].nimi ) do begin
    Caption := s;
   end;
  kentat[n].edit := TDBEdit.Create(self);                               { komp }
  with ( kentat[n].edit ) do begin
    DataSource := self.DataSource;                                      { komp }
    DataField := s;
//    Parent := self;
   end;                                                                 { komp }
  kentat[n].index := n;
  Paikka(kentat[n]);
  kentat[n].nimi.Parent := self;  { Vhent vlkkymist kun tehdn vasta tll }
  kentat[n].edit.Parent := self;
  inc(n);
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.LisaaKaikki;
var i:integer;
begin
  Siivoa;
  FWorking := true;                                                     { komp }
  if ( DataSource = nil ) or ( DataSource.DataSet = nil ) then exit;    { komp }
  for i:=0 to DataSource.DataSet.FieldCount-1 do
    Lisaa(DataSource.DataSet.Fields[i].FieldName);
  FWorking := false;                                                    { komp }
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.SetEditLeveys(l:integer);
begin
  edit_leveys := l;
  MuutaKoko;
end;

{------------------------------------------------------------------------------}
procedure TDynKentat.SetNimenLeveys(l:integer);
begin
  nimen_leveys := l;
  MuutaKoko;
end;

{------------------------------------------------------------------------------}
{ Komponenttina olemista varten listyt metodit                         { komp }
{------------------------------------------------------------------------------}
procedure TDynKentat.SetKentanKorkeus(h:integer);
begin
  kentan_korkeus := h;
  MuutaKoko;
end;

procedure TDynKentat.SetDataSource(const Value: TDataSource);
begin
  Siivoa;
  if ( DataSource <> nil ) and ( DataSource.DataSet <> nil ) then begin
    DataSource.DataSet.AfterOpen := FOldAfter;
    DataSource.DataSet.BeforeClose := FOldBefore;
  end;
  FOldAfter := nil;
  FOldBefore := nil;
  FDataSource := Value;
  if ( DataSource = nil ) then exit;
  if ( DataSource.DataSet <> nil ) then begin
    FOldAfter := DataSource.DataSet.AfterOpen;
    FOldBefore := DataSource.DataSet.BeforeClose;
    DataSource.DataSet.BeforeClose := BeforeClose;
    DataSource.DataSet.AfterOpen := AfterOpen;
  end;
  LisaaKaikki;
end;

procedure TDynKentat.AfterOpen(DataSet: TDataSet);
begin
  LisaaKaikki;
  if ( Assigned(FOldAfter) ) then FOldAfter(DataSet);
end;

procedure TDynKentat.BeforeClose(DataSet: TDataSet);
begin
  Siivoa;
  if ( Assigned(FOldBefore) ) then FOldBefore(DataSet);
end;

procedure TDynKentat.HandleResize;
begin
  if ( Parent = nil ) then exit;
  FWorking := true;
  EditLeveys := ClientWidth-RightMargin-NimenLeveys-20;
  FWorking := false;
end;

procedure TDynKentat.SetRightMargin(const Value: integer);
begin
  FRightMargin := Value;
  HandleResize;
end;

procedure TDynKentat.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited;
  if ( not FWorking ) then HandleResize;
end;

procedure TDynKentat.Loaded;
begin
  inherited;
  DataSource := DataSource;
end;

end.
